@@ -23,30 +23,36 @@ summary.hmm <- function (object, ...)
 }
 
 
-simulate.hmmspec <- function(object, nsim, seed=NULL, ...) {
+simulate.hmmspec <- function(object, nsim, seed=NULL, r=NULL,...) {
 
   if(!is.null(seed)) set.seed(seed)
+  if(is.null(r)&is.null(object$r)) stop("r not specified")
+  if(is.null(r)) r=object$r
+  
   if(length(nsim)==1) {
    s1 = sim.mc(object$init,object$transition, nsim)
-   x = sapply(s1, object$r, model=object)
-   ret = list(s = s1, x = x, N = nsim)
+   x = sapply(s1, r, model=object)
+    if (NCOL(x) > 1) 
+        ret = list(s = s1, x = t(x), N = nsim)
+    else ret = list(s = s1, x = x, N = nsim)
    class(ret) <- "hsmm.data"
    ret
   }
-  else  .sim.mhmm(object,nsim)
+  else  .sim.mhmm(object,nsim,r)
 }
 
-.sim.mhmm <- function(model,N) {
+.sim.mhmm <- function(model,N,r) {
   s1 = sim.mc(model$init,model$transition,N)#the hidden states
-  x = sapply(s1,model$r,model) #simulate the observed state sequence
-  ret = list(s=s1,x=x,N=N)
+  x = sapply(s1,r,model) #simulate the observed state sequence
+  if(NCOL(x)>1) ret = list(s=s1,x=t(x),N=N)
+  else ret = list(s=s1,x=x,N=N)
+
   class(ret) <- "mhsmm.data"
   ret                                            
 }
 
-
-hmmspec <- function(init, trans, emission,r=rnorm.hsmm,f=dnorm.hsmm) {
- ans <- list(J=length(init), init = init, transition = trans, emission = emission,r=r,f=f)
+hmmspec <- function(init, trans, emission,f,r=NULL,mstep=NULL) {
+ ans <- list(J=length(init), init = init, transition = trans, emission = emission,f=f,r=r,mstep=mstep)
  class(ans) <- "hmmspec"
  return(ans)
 }
@@ -57,7 +63,17 @@ print.hmm <- function(x, ...) {
  return(invisible(x))
 }
 
-hmmfit <- function(x,start.val,f=dnorm.hsmm,mstep=mstep.norm,tol=1e-08,maxit=1000) 
+.estep.hmm <- function(x,object) {
+    K=nrow(object$model$transition)
+    model=object$model
+    N=x$N
+    p = sapply(1:K,fn <- function(state) object$f(x$x,state,model))
+    tmp = .C("mo_estep_hmm",a=as.double(t(model$transition)),pi=as.double(t(model$init)),p=as.double(t(p)),N=as.integer(x$N),nsequences=as.integer(length(x$N)),
+      K=as.integer(K),gam=double(K*sum(N)),ll=double(1),PACKAGE='mhsmm')      
+    list(gamma=matrix(tmp$gam,ncol=K),loglik=tmp$ll)
+}
+
+hmmfit <- function(x,start.val,mstep=mstep.norm,lock.transition=FALSE,tol=1e-08,maxit=1000) 
 {
   model = start.val
   K = nrow(model$trans)
@@ -69,7 +85,7 @@ hmmfit <- function(x,start.val,f=dnorm.hsmm,mstep=mstep.norm,tol=1e-08,maxit=100
 	  N = NN = x$N
   	x = x$x
   }
-  
+
   if(K<2) stop("K must be larger than one.")	
   if(any(dim(model$trans)!=K)) stop("dimensions of a incorrect")
   if(length(model$init)!=K) stop("dimensions of st incorrect")
@@ -77,6 +93,12 @@ hmmfit <- function(x,start.val,f=dnorm.hsmm,mstep=mstep.norm,tol=1e-08,maxit=100
   if(length(N)==1) NN=1
   else NN = length(N)
 
+ if(is.null(mstep)) 
+    if(is.null(model$mstep)) stop("mstep not specified")
+    else  mstep=model$mstep      
+
+  f=model$f
+
   loglik=numeric(maxit)
   loglik=NA
   loglik[1]=-Inf  
@@ -93,8 +115,10 @@ hmmfit <- function(x,start.val,f=dnorm.hsmm,mstep=mstep.norm,tol=1e-08,maxit=100
 #    if((loglik[i]-loglik[i-1])<(-tol)) stop(paste("loglikelihood has decreased on iteration",i))
     gam = matrix(test$gam,ncol=K)
     model$emission = mstep(x,gam)
-    model$transition=matrix(test$a,nrow=K,byrow=TRUE)
-    model$init=test$pi
+    if(!lock.transition) {
+      model$transition=matrix(test$a,nrow=K,byrow=TRUE)
+      model$init=test$pi
+    }
   }  
   ret = list(model=model,K=K,f=f,mstep=mstep,gam=gam,loglik=loglik[!is.na(loglik)],N=N,p=gam,yhat=apply(gam,1,which.max))
   class(ret) <- "hmm"
@@ -103,20 +127,17 @@ hmmfit <- function(x,start.val,f=dnorm.hsmm,mstep=mstep.norm,tol=1e-08,maxit=100
 
 
 predict.hmm <- function(object,x,method="viterbi",...) {
-  nseq=1  
   if(class(x)=="numeric" | class(x)=="integer") {
   	warning('x is a primitive vector.  Assuming single sequence.')
   	N = NROW(x)
-  	NN = c(0,N)
   	if(N<1) stop("N less than one")
+  	x=list(x=x,N=NROW(x))
   }
-  else{
+  nseq=length(x$N) 
   	N = x$N
   	NN = cumsum(c(0,x$N))
-  	x0 = x$x
-  }
-#   state = integer(obj
   if(method=="viterbi") {
+    nseq=1
     K = object$K
     p = sapply(1:K,fn <- function(state) object$f(x$x,state,object$model))
     p[p==0]= 1e-200
@@ -134,13 +155,13 @@ predict.hmm <- function(object,x,method="viterbi",...) {
     ans <- list(s=state,x=x$x,N=x$N)
   }
   else if(method=="smoothed") {
-    tmp <- hmmfit(x,object$model,object$f,object$mstep,maxit=1)
-    ans <- list(s=tmp$yhat,x=x$x,N=x$N,p=tmp$p)
+    tmp <- .estep.hmm(x,object)
+    yhat <- apply(tmp$gamma,1,which.max)
+    ans <- list(s=yhat,x=x$x,N=x$N,p=tmp$gamma,loglik=tmp$loglik)
   }
   else stop("Unavailable prediction method")
   class(ans) <- "hsmm.data"
   ans
-#  void viterbi(double *a,double *start,double *p,int *T,int *nsequences,int *nstates,int *q) {
 }
 
 
